#!/usr/bin/env python
## Copyright (c) 2010 Marnix H. Medema
## University of Groningen
## Department of Microbial Physiology / Groningen Bioinformatics Centre

###INSTRUCTIONS:
###COMPATIBLE WITH BOTH LINUX AND WINDOWS
###TO RUN BATCH ANTISMASH RUNS, PLACE THIS SCRIPT IN YOUR ANTISMASH MAIN FOLDER.
###THEN, CREATE A SUBFOLDER 'batch_input' THERE WITH THE FILES YOU WANT TO ANALYZE.
###ALTERNATIVELY, SUPPLY A SINGLE MULTI-GBK/EMBL FILE AS ARGUMENT FROM THE COMMAND LINE
###THE OUTPUT FILES WILL BE COPIED TO A FOLDER NAMED 'batch_output'.
###WARNING: THE FILES SAVED IN THE INPUTFILES FOLDER MAY NOT CONTAIN SPACES OR SPECIAL CHARACTERS (!"#$%&()*+,./:;=>?@[]^`"{|}). OTHERWISE, THE RUN WILL FAIL.
###ALSO, REMOVE OR RENAME ALL ANTISMASH OUTPUT FOLDERS WITH THE SAME NAMES AS THE FILES IN THE 'batch_input' FOLDER.
###USAGE: python batch_antismash.py (inputfile.gbk) --options

##Script for batch-processing a large number of files in antiSMASH
#List all input files
import os
import sys
import shutil

if "batch_input" not in os.listdir("."):
  os.mkdir("batch_input")

#Set Python environment to antiSMASH python installation version (Windows / Mac)
if sys.platform != ('linux2'):
  os.environ['PYTHON'] = os.getcwd() + os.sep + "python"

#If multi-GBK/multi-EMBL/multi-FASTA file is supplied, split file and copy subfiles to batch_input
if len(sys.argv) > 1 and sys.argv[1].rpartition(".")[2] in ["gbk","GBK","gb","GB","genbank","GENBANK","embl","EMBL","emb","EMB","fasta","FASTA","fas","FAS","fa","FA"]:
  try:
    if sys.argv[1].rpartition(".")[2] in ["gbk","GBK","gb","GB","genbank","GENBANK"]:
      gbk_entries = ["LOCUS       " + entry for entry in open(sys.argv[1],"r").read().split("LOCUS       ")[1:]]
      entrynr = 1
      for entry in gbk_entries:
        entryfile = open("batch_input" + os.sep + sys.argv[1].rpartition(".")[0] + "_entry" + str(entrynr) + "." + sys.argv[1].rpartition(".")[2],"w")
        entryfile.write(entry)
        entryfile.close()
        entrynr += 1
    elif sys.argv[1].rpartition(".")[2] in ["embl","EMBL","emb","EMB"]:
      embl_entries = ["ID   " + entry for entry in open(sys.argv[1],"r").read().split("ID   ")[1:]]
      entrynr = 1
      for entry in embl_entries:
        entryfile = open("batch_input" + os.sep + sys.argv[1].rpartition(".")[0] + "_entry" + str(entrynr) + "." + sys.argv[1].rpartition(".")[2],"w")
        entryfile.write(entry)
        entryfile.close()
        entrynr += 1
    elif sys.argv[1].rpartition(".")[2] in ["fasta","FASTA","fas","FAS","fa","FA"]:
      fasta_entries = ["\n>" + entry for entry in open(sys.argv[1],"r").read().split("\n>")]
      entrynr = 1
      for entry in fasta_entries:
        entryfile = open("batch_input" + os.sep + sys.argv[1].rpartition(".")[0] + "_entry" + str(entrynr) + "." + sys.argv[1].rpartition(".")[2],"w")
        entryfile.write(entry)
        entryfile.close()
        entrynr += 1
  except:
    print "Failed to parse multi-GBK / multi-EMBL  / multi-FASTA file. Exiting..."
    sys.exit()
  sys.argv = sys.argv[2:]
else:
  sys.argv = sys.argv[1:]

#Find input files
os.chdir("batch_input")
dirfiles = os.listdir(".")
inputfiles = ["batch_input" + os.sep + i for i in dirfiles if (".fasta" in i or ".FASTA" in i or ".fas" in i or ".FAS" in i or ".fa" in i or ".FA" in i or ".gbk" in i or ".GBK" in i or ".gb" in i or ".GB" in i or ".embl" in i or ".EMBL" in i or ".emb" in i or ".EMB" in i)]
if len(inputfiles) == 0:
  print "No files selected for analysis. Exiting..."
  sys.exit()
os.chdir("..")

#Copy each file to running folder, run and copy results
try:
  os.system("mkdir batch_output")
except:
  pass

#Run antiSMASH for each file; copy to batch_output folder; clean up
for j in inputfiles:
  if j.rpartition(os.sep)[2].rpartition(".")[0] in os.listdir("."):
    overwrite = raw_input("Output folder '" + j.rpartition(os.sep)[2].rpartition(".")[0] + "' exists in antiSMASH directory. Overwrite? (y/n)")
    while overwrite != "y" and overwrite != "n":
      overwrite = raw_input("Overwrite? (y/n)")
    if overwrite == "n":
      print "Skipping file..."
      break
    elif overwrite == "y":
      shutil.rmtree(j.rpartition(os.sep)[2].rpartition(".")[0])
  #Copy file to antiSMASH folder
  shutil.copyfile(j,j.rpartition(os.sep)[2])
  j = j.rpartition(os.sep)[2]
  #Run antiSMASH
  if sys.platform == ('win32'):
    os.system("antismash " + j + " " + " ".join(sys.argv))
  elif sys.platform == ('linux2') or sys.platform == ('darwin'):
    os.system("python antismash.py " + j + " " + " ".join(sys.argv))
  #Copy results folder and remove initial output files
  if j.rpartition(".")[0] in os.listdir("batch_output"):
    shutil.rmtree("batch_output" + os.sep + j.rpartition(".")[0])
  shutil.copytree(j.rpartition(".")[0], "batch_output" + os.sep + j.rpartition(".")[0])
  shutil.rmtree(j.rpartition(".")[0])
  os.remove(j)
  
#Create an overview file with the results from all files
overviewfile = open("batch_output" + os.sep + "output_overview.txt","w")
for j in inputfiles:
  source = j.rpartition(os.sep)[2].rpartition(".")[0]
  if "geneclusters.txt" in os.listdir("batch_output" + os.sep + source):
    txtfile = open("batch_output" + os.sep + source + os.sep + "geneclusters.txt","r")
    for line in [line for line in txtfile.read().split("\n") if line != ""]:
      overviewfile.write(line + '\n')
    txtfile.close()
overviewfile.close()